/* Copyright (c) 2003 The Nutch Organization.  All rights reserved.   */
/* Use subject to the conditions in http://www.nutch.org/LICENSE.txt. */

package net.nutch.searcher;

import cgl.cluster.*;
import java.io.IOException;
import java.io.File;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.net.InetSocketAddress;

import java.util.logging.Logger;

import javax.servlet.ServletContext;

import net.nutch.util.LogFormatter;
import net.nutch.util.NutchConf;


/** One stop shopping for search-related functionality. */
public class NutchBean
  implements Searcher, HitDetailer, HitSummarizer, HitContent {

 // public static final Logger LOG =
 //   LogFormatter.getLogger("net.nutch.searcher.NutchBean");

  static {
   // LogFormatter.setShowThreadIDs(true);
  }

  private String[] segmentNames;

  private Searcher searcher;
  private HitDetailer detailer;
  private HitSummarizer summarizer;
  private HitContent content;
  private String[] summary;

  /** Cache in servlet context. */
  public static NutchBean get(ServletContext app) throws IOException {
    NutchBean bean = (NutchBean)app.getAttribute("nutchBean");
    if (bean == null) {
     // LOG.info("creating new bean");
      bean = new NutchBean();
      app.setAttribute("nutchBean", bean);
    }
    return bean;
  }

  /** Construct reading from connected directory. */
  public NutchBean() throws IOException {
    this(new File(NutchConf.get("searcher.dir", ".")));
    System.out.println("CURRENT" + NutchConf.get("searcher.dir"));
  }

  /** Construct in a named directory. */
  public NutchBean(File dir) throws IOException {
    File servers = new File(dir, "search-servers.txt");
    if (servers.exists()) {
//      LOG.info("searching servers in " + servers.getCanonicalPath());
      init(new DistributedSearch.Client(servers));
    } else {
      init(new File(dir, "index"), new File(dir, "segments"));
    }
  }

  private void init(File indexDir, File segmentsDir) throws IOException {
    IndexSearcher indexSearcher;
    if (indexDir.exists()) {
//      LOG.info("opening merged index in " + indexDir.getCanonicalPath());
      indexSearcher = new IndexSearcher(indexDir.toString());
    } else {
//      LOG.info("opening segment indexes in " + segmentsDir.getCanonicalPath());
      indexSearcher = new IndexSearcher(segmentsDir.listFiles());
    }

    FetchedSegments segments = new FetchedSegments(segmentsDir.toString());

    this.segmentNames = segments.getSegmentNames();

    this.searcher = indexSearcher;
    this.detailer = indexSearcher;
    this.summarizer = segments;
    this.content = segments;
  }

  private void init(DistributedSearch.Client client) throws IOException {
    this.segmentNames = client.getSegmentNames();
    this.searcher = client;
    this.detailer = client;
    this.summarizer = client;
    this.content = client;
  }


  public String[] getSegmentNames() {
    return segmentNames;
  }

  public Hits search(Query query, int numHits) throws IOException {
    return searcher.search(query, numHits);
  }

  public float[] score(Query query, int numHits) throws IOException {
    return searcher.score(query, numHits);
  }

  public String getExplanation(Query query, Hit hit) throws IOException {
    return searcher.getExplanation(query, hit);
  }

  public HitDetails getDetails(Hit hit) throws IOException {
    return detailer.getDetails(hit);
  }

  public HitDetails[] getDetails(Hit[] hits) throws IOException {
    return detailer.getDetails(hits);
  }

  public String getSummary(HitDetails hit, Query query) throws IOException {
    return summarizer.getSummary(hit, query);
  }

  public String[] getSummary(HitDetails[] hits, Query query)
    throws IOException {
    return summarizer.getSummary(hits, query);
  }

  public byte[] getContent(HitDetails hit) throws IOException {
    return content.getContent(hit);
  }

  public String[] getAnchors(HitDetails hit) throws IOException {
    return content.getAnchors(hit);
  }

  public void setSum( String[] sum) {
    summary = sum;
  }

  public String[] getSum() {
    return summary;
  }


  /** For debugging. */
  public static void main(String[] args) throws Exception {
    String usage = "NutchBean query";

    int numHits = 100;
    NutchBean bean = new NutchBean();
    Hit[] new_Hits1 = null;
    Hit[] new_Hits2 = null;
    Query query = Query.parse(args[0]);
    System.out.println("QUEry.." + query);
    Hits hits1 = bean.search(query, 10);
    System.out.println("RESULTS WITHOUT PR ");
    for (int j = 0; j < hits1.getLength(); j++) {
      HitDetails detail = bean.getDetails(hits1.getHit(j));
      String url = detail.getValue("url");
      Vector vec = detail.getPagerank(1);
      System.out.println(" " + j + " " + url + " " + "pagerank = " + vec);
    }

/*

    if (args.length == 0) {
      System.err.println(usage);
      System.exit(-1);
    }
 // Set number of Hits in order to search from web database
    int numHits = 100;
    NutchBean bean = new NutchBean();
    Hit[] new_Hits1 = null;
    Hit[] new_Hits2 = null;
    Query query = Query.parse(args[0]);
    Hits hits1 = bean.search(query, 1);
    int length = (int)Math.min(hits1.getTotal(), numHits);
    Hits hits = bean.search(query, length);
    System.out.println("Total hits: " + hits.getTotal());
    float[] scores = bean.score(query, length);
    for(int i=0; i<scores.length; i++)
      System.out.println(i + " " + scores[i]);

    Hit[] HITS = hits.getHits(0, length); //(int)hits.getTotal());
    System.out.println("RESULTS WITHOUT PR ");
    for (int j = 0; j < HITS.length; j++) {
      HitDetails detail = bean.getDetails(HITS[j]);
      String url = detail.getValue("url");
      System.out.println(" " + j + " " + url);
    }

    PreferenceCluster pc = PreferenceCluster.instance();
    Hashtable table = pc.readPRfromFile();
    HitScore[] HS = pc.getPRList(HITS, scores, table);
    Hit[] PRHits = new Hit[HS.length];
    for (int i = 0; i < HS.length; i++) {
      PRHits[i] = HS[i].getHit();
    }

    System.out.println("RESULTS WITH NUTCH");
    System.out.println("SIZE...." + PRHits.length);
    for (int j = 0; j < PRHits.length; j++) {
      HitDetails detail = pc.getDetails(PRHits[j]);
      String url = detail.getValue("url");
      System.out.println(" " + j + " " + url);
    }


  try {
    System.out.println("RESULTS WITH PLAIN PAGERANK");
    Vector pref = new Vector();
    HitScore[] newHS = pc.getPRHitList(HS, pref, table);
    new_Hits1 = new Hit[newHS.length];
    for( int i = 0; i < newHS.length; i++)
      new_Hits1[i] = newHS[i].getHit();
    HitDetails detail = bean.getDetails(new_Hits1[0]);
    System.out.println("URL to Browser " + detail.getValue("url"));
    System.out.println("SIZE..." + new_Hits1.length);
    for (int k = 0; k < new_Hits1.length; k++) {
      HitDetails details = pc.getDetails(new_Hits1[k]);
      String url = details.getValue("url");
      System.out.println(" " + k + " " + url);
    }

  }
  catch (Exception ex) {
    ex.printStackTrace();
  }

  try {
    Vector vec = new Vector();
    vec.add("gov");
    vec.add("org");
    System.out.println("RESULTS WITH WEIGHTED PAGERANK with combination of " + vec);
    HitScore[] newHS2 = pc.getPRHitList(HS, vec, table);
    new_Hits2 = new Hit[newHS2.length];
    for( int i = 0; i < newHS2.length; i++)
      new_Hits2[i] = newHS2[i].getHit();
    for (int t = 0; t < new_Hits2.length; t++) {
      HitDetails detail = pc.getDetails(new_Hits2[t]);
      String url = detail.getValue("url");
      System.out.println(" " + t + " " + url);
    }

  }
  catch (Exception ex) {
    ex.printStackTrace();
  }
// Gets combined data from 3 different approaches, spread over the array randomly.
  Hit[] last = pc.Combine( PRHits, new_Hits1, new_Hits2);
  for( int i = 0; i < last.length; i++)
    System.out.println("Scores ... " + last[i].getScore());
*/
  }



}
